/*
FRODO: a FRamework for Open/Distributed Optimization
Copyright (C) 2008, 2009, 2010, 2011  Thomas Leaute, Brammert Ottens & Radoslaw Szymanek

FRODO is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

FRODO is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.


How to contact the authors: 
<http://lia.epfl.ch/>

EPFL / IC / IIF / LIA
Batiment IN 
Station 14 
CH - 1015 Lausanne 
Switzerland
*/

/**
 * 
 */
package ch.epfl.lia.frodo.daemon.userIO;

import java.io.IOException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Collection;

import ch.epfl.lia.frodo.communication.AgentAddress;
import ch.epfl.lia.frodo.communication.IncomingMsgPolicyInterface;
import ch.epfl.lia.frodo.communication.Message;
import ch.epfl.lia.frodo.communication.Queue;
import ch.epfl.lia.frodo.communication.QueueOutputPipeInterface;
import ch.epfl.lia.frodo.controller.messages.MessageAgentReporting;
import ch.epfl.lia.frodo.daemon.Daemon;
import ch.epfl.lia.frodo.daemon.LocalWhitePages;
import ch.epfl.lia.frodo.controller.Controller;
import ch.epfl.lia.frodo.controller.WhitePages;
import ch.epfl.lia.frodo.controller.ConfigurationManager;

/**
 * @author brammertottens
 *
 */
public abstract class UserIO extends Thread implements IncomingMsgPolicyInterface<String> {
	
	/** The daemon this UI belongs to*/
	Daemon daemon;
	
	/** the daemons queue*/
	Queue queue;
	
	/** The list of messages types this listener wants to be notified of */
	private ArrayList <String> msgTypes = new ArrayList <String> ();

	/**
	 * Constructor
	 * @param daemon 	the daemon
	 */
	public UserIO(Daemon daemon) {
		this.daemon = daemon;
		msgTypes.add(WhitePages.KILL_AGENTS);
		msgTypes.add(ConfigurationManager.AGENT_CONFIGURATION_MESSAGE);
	}

	/** @see java.lang.Thread#start() */
	@Override
	public void start () {
		super.start();
	}

	/** 
	 * @see ch.epfl.lia.frodo.communication.IncomingMsgPolicyInterface#getMsgTypes()
	 */
	public Collection<String> getMsgTypes() {
		return msgTypes;
	}

	/** 
	 * @see IncomingMsgPolicyInterface#notifyIn(Message)
	 */
	public void notifyIn(Message msg) {
		/// @todo Auto-generated method stub
		return;
	}

	/** 
	 * @see ch.epfl.lia.frodo.communication.IncomingMsgPolicyInterface#setQueue(ch.epfl.lia.frodo.communication.Queue)
	 */
	public void setQueue(Queue queue) {
		this.queue = queue;
	}
	
	/**
	 * Report the daemon to the controller
	 * @param address of the controller
	 * @author Brammert Ottens, Thomas Leaute
	 */
	public void registerController(AgentAddress address) {
		QueueOutputPipeInterface output = null;
		try {
			output = Controller.PipeFactoryInstance.outputPipe(address);
		} catch (UnknownHostException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		queue.addOutputPipe(LocalWhitePages.CONTROLLER_ID, output);
		MessageAgentReporting msg = new MessageAgentReporting(MessageAgentReporting.DEAMON_REPORTING, daemon.daemonId, daemon.daemonAddress);
		queue.sendMessage(LocalWhitePages.CONTROLLER_ID, msg);
	}
	
	/**
	 * Quit the daemon;
	 */
	public void exit() {
		daemon.exit(true);
	}
	
	/**
	 * Give a message to the user
	 * @param message 	the message to be presented to the user
	 */
	public abstract void tellUser(String message);
	
	/**
	 * Stop the user interface
	 */
	public abstract void stopRunning();

}
